from pathlib import Path

from voicemailbox import logging


__version__ = '0.1.2'

# Activate logs
logger = logging.Logging().logger

# Define constants
STATIC_FILES_PATH = Path(__file__).parent / "static"
NEW_MAIL_SONG_PATH = STATIC_FILES_PATH / "songs" / "new_mail.mp3"
FONTAWESOME_PATH = (
        STATIC_FILES_PATH / "fonts" / "Font Awesome 5 Free-Solid-900.otf")
SLIDER_BORDER_PATH = STATIC_FILES_PATH / "images" / "blank_background.png"
SLIDER_CURSOR_PATH = STATIC_FILES_PATH / "images" / "circle-solid-24.png"



from kivy import app as kv_app
from kivy import clock as kv_clock

from voicemailbox import screens


class VoicemailBoxApp(kv_app.App):

    def build(self):
        root_screen = screens.RootScreen()

        # Loop to update position of song every 5 ms
        kv_clock.Clock.schedule_interval(
            callback=lambda x: root_screen.update_position(
                root_screen.voicemail_screen_1),
            timeout=0.05,
        )
        kv_clock.Clock.schedule_interval(
            callback=lambda x: root_screen.update_position(
                root_screen.voicemail_screen_2),
            timeout=0.05,
        )

        # Loop to update new voicemails list every second
        kv_clock.Clock.schedule_interval(
            callback=root_screen.update_new_voicemails,
            timeout=1,
            )

        # Loop to update date and time every second
        kv_clock.Clock.schedule_interval(
            callback=root_screen.home_screen.update_datetime,
            timeout=1,
        )

        return root_screen



from kivy import animation as kv_animation


class Animation:
    """
    Class gathering all transition methods
    """
    @staticmethod
    def background_fade(widget, background_color):
        """
        Background fading
        """
        fade = kv_animation.Animation(background_rgba=background_color,
                                      duration=1,
                                      t="in_back")
        return fade

    @staticmethod
    def fade_out_in(widget, attr_name, attr_value, **kwargs):
        """
        Widget fading out, widget attribute change and widget fading in
        """
        def update_during_transition(_widget, _attr_name, _attr_value):
            setattr(_widget, _attr_name, _attr_value)
            fade_in = kv_animation.Animation(opacity=1,
                                             duration=0.5,
                                             t="in_back")
            fade_in.start(widget)

        fade_out = kv_animation.Animation(opacity=0, duration=0.5, t="in_back")
        fade_out.bind(
            on_complete=lambda *args: update_during_transition(widget,
                                                               attr_name,
                                                               attr_value))
        return fade_out

    @staticmethod
    def fade_in(widget):
        """
        Widget fading in
        """
        def enable(_widget):
            _widget.disabled = False

        fade_in = kv_animation.Animation(opacity=1, duration=1, t="in_back")
        fade_in.bind(on_start=lambda *args: enable(widget))
        return fade_in

    @staticmethod
    def fade_out(widget):
        """
        Widget fading out
        """
        def disable(_widget):
            _widget.disabled = True

        fade_out = kv_animation.Animation(opacity=0, duration=1, t="in_back")
        fade_out.bind(on_complete=lambda *args: disable(widget))
        return fade_out
